@@ -1,4 +1,4 @@
-/* Copyright (C) 2016 J.F.Dockes
+/* Copyright (C) 2016-2023 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser General Public License as published by
  *   the Free Software Foundation; either version 2.1 of the License, or
@@ -20,9 +20,10 @@
 
 #include <string>
 #include <vector>
-#include <sstream>
 #include <functional>
 #include <memory>
+#include <mutex>
+#include <sstream>
 
 #include <string.h>
 #include <fcntl.h>
@@ -45,28 +46,46 @@
 
 using namespace std;
 using namespace std::placeholders;
-using namespace UPnPProvider;
 
 class StreamHandle {
 public:
-    StreamHandle(PlgWithSlave::Internal *plg) {
-    }
-    ~StreamHandle() {
-        clear();
-    }
+    StreamHandle() = default;
+    ~StreamHandle() = default;
     void clear() {
-        plg = 0;
         path.clear();
         media_url.clear();
         len = 0;
         opentime = 0;
     }
     
-    PlgWithSlave::Internal *plg;
     string path;
     string media_url;
-    long long len;
-    time_t opentime;
+    long long len{0};
+    time_t opentime{0};
+};
+
+class ContentCacheEntry {
+public:
+    int toResult(int stidx, int cnt, vector<UpSong>& entries) const;
+    time_t m_time{time(0)};
+    int m_offset{0};
+    int m_total{-1};
+    vector<UpSong> m_results;
+};
+
+class ContentCache {
+public:
+    ContentCache(int retention_secs = 300)
+        : m_retention_secs(retention_secs) {}
+    std::unique_ptr<ContentCacheEntry> get(const string& query);
+    ContentCacheEntry& set(const string& query, ContentCacheEntry &entry);
+
+private:
+    time_t m_lastpurge{time(0)};
+    int m_retention_secs;
+    unordered_map<string, ContentCacheEntry> m_cache;
+
+    void purge();
 };
 
 // Timeout seconds for reading data from plugins. Be generous because
@@ -77,16 +96,16 @@ static const int read_timeout(60);
 class PlgWithSlave::Internal {
 public:
     Internal(PlgWithSlave *_plg)
-        : plg(_plg), cmd(read_timeout), laststream(this) {
+        : plg(_plg), cmd(read_timeout) {
 
         string val;
-        if (g_config->get("plgproxymethod", val) && !val.compare("proxy")) {
+        if (getOptionValue("plgproxymethod", val) && !val.compare("proxy")) {
             doingproxy = true;
         }
 #ifdef ENABLE_SPOTIFY
         if (!plg->getname().compare("spotify")) {
-            g_config->get("spotifyuser", user);
-            g_config->get("spotifypass", password);
+            getOptionValue("spotifyuser", user);
+            getOptionValue("spotifypass", password);
             string cachedir = path_cat(g_cachedir, "spotify");
             // Make sure doingproxy is set, independantly of the
             // config variable, which is only useful for Qobuz et al
@@ -101,6 +120,7 @@ public:
     }
     bool maybeStartCmd();
 
+    std::mutex mutex;
     PlgWithSlave *plg;
     CmdTalk cmd;
     bool doingproxy{false};
@@ -113,6 +133,10 @@ public:
     
     // Cached uri translation
     StreamHandle laststream;
+    // Cache for searches
+    ContentCache scache{300};
+    // Cache for browsing
+    ContentCache bcache{180};
 };
 
 // HTTP Proxy/Redirect handler
@@ -120,17 +144,17 @@ static StreamProxy *o_proxy;
 
 StreamProxy::UrlTransReturn translateurl(
     CDPluginServices *cdsrv,
+    const std::string& useragent,
     std::string& url,
     const std::unordered_map<std::string, std::string>& querymap,
     std::unique_ptr<NetFetch>& fetcher
     )
 {
-    LOGDEB("PlgWithSlave::translateurl: url " << url << endl);
+    LOGDEB("PlgWithSlave::translateurl: url " << url << "\n");
 
-    PlgWithSlave *realplg =
-        dynamic_cast<PlgWithSlave*>(cdsrv->getpluginforpath(url));
+    PlgWithSlave *realplg = dynamic_cast<PlgWithSlave*>(cdsrv->getpluginforpath(url));
     if (nullptr == realplg) {
-        LOGERR("PlgWithSlave::translateurl: no plugin for path ["<<url<< endl);
+        LOGERR("PlgWithSlave::translateurl: no plugin for path ["<<url<< "\n");
         return StreamProxy::Error;
     }
 
@@ -143,13 +167,13 @@ StreamProxy::UrlTransReturn translateurl(
     // The uprcl module has a real path and no trackid. Handle both cases
     const auto it = querymap.find("trackId");
     if (it != querymap.end() && !it->second.empty()) {
-        path += string("?version=1&trackId=") + it->second;
+        path += string("/version/1/trackId/") + it->second;
     }
 
     // Translate to Tidal/Qobuz etc real temporary URL
-    url = realplg->get_media_url(path);
+    url = realplg->get_media_url(path, useragent);
     if (url.empty()) {
-        LOGERR("answer_to_connection: no media_uri for: " << url << endl);
+        LOGERR("answer_to_connection: no media_uri for: " << url << "\n");
         return StreamProxy::Error;
     }
     StreamProxy::UrlTransReturn method = realplg->doproxy() ?
@@ -175,23 +199,39 @@ StreamProxy::UrlTransReturn translateurl(
 // not need to generate URLs
 bool PlgWithSlave::startPluginCmd(CmdTalk& cmd, const string& appname,
                                   const string& host, unsigned int port,
-                                  const string& pathpref)
+                                  const string& pathpref, string upnphost, int upnpport)
 {
     string pythonpath = string("PYTHONPATH=") +
         path_cat(g_datadir, "cdplugins") + ":" +
         path_cat(g_datadir, "cdplugins/pycommon") + ":" +
         path_cat(g_datadir, "cdplugins/" + appname);
     string configname = string("UPMPD_CONFIG=") + g_configfilename;
-    stringstream ss;
-    ss << host << ":" << port;
-    string hostport = string("UPMPD_HTTPHOSTPORT=") + ss.str();
+
+    // Send the microhttpd host:port and pathprefix strings through the environment. Used by
+    // plugins which need to redirect or proxy their URLs (e.g. Qobuz, or, previously, now
+    // obsolete Tidal, Spotify...) This allows the plugin to construct appropriate URLs. The
+    // pathprefix is used to determine which plugin an URL belongs to.
+    // The URLs are like: http://$UPMPD_HTTPHOSTPORT/$UPMPD_PATHPREFIX/...
+    
+    string hostport = string("UPMPD_HTTPHOSTPORT=") + host + ":" + ulltodecstr(port);
     string pp = string("UPMPD_PATHPREFIX=") + pathpref;
+
+    std::vector<std::string> env{pythonpath, configname, hostport, pp};
+
+    // Send the UPnP (libnpupnp) HTTP IP address and port. This allows using the internal
+    // lib(np)upnp HTTP server to serve local files (if enabled by setting webserverdocumentroot in
+    // the config accordingly, see main.hxx).
+    if (!g_npupnpwebdocroot.empty()) {
+        env.push_back(string("UPMPD_UPNPHOSTPORT=") + upnphost + ":" + lltodecstr(upnpport));
+        env.push_back(string("UPMPD_UPNPDOCROOT=") + g_npupnpwebdocroot);
+    }
+
+    env.push_back(string("UPMPD_PKGDATADIR=") + g_datadir);
+
     string exepath = path_cat(g_datadir, "cdplugins");
     exepath = path_cat(exepath, appname);
-    exepath = path_cat(exepath, appname + "-app" + ".py");
-
-    if (!cmd.startCmd(exepath, {/*args*/},
-                      /* env */ {pythonpath, configname, hostport, pp})) {
+    exepath = path_cat(exepath, appname + "-app.py");
+    if (!cmd.startCmd(exepath, {/*args*/}, env)) {
         LOGERR("PlgWithSlave::maybeStartCmd: startCmd failed\n");
         return false;
     }
@@ -205,9 +245,7 @@ bool PlgWithSlave::maybeStartProxy(CDPluginServices *cdsrv)
 {
     if (nullptr == o_proxy) {
         int port = CDPluginServices::microhttpport();
-        o_proxy = new StreamProxy(
-            port,
-            std::bind(&translateurl, cdsrv, _1, _2, _3));
+        o_proxy = new StreamProxy(port, std::bind(&translateurl, cdsrv, _1, _2, _3, _4));
             
         if (nullptr == o_proxy) {
             LOGERR("PlgWithSlave: Proxy creation failed\n");
@@ -231,7 +269,10 @@ bool PlgWithSlave::Internal::maybeStartCmd()
     if (!startPluginCmd(cmd, plg->m_name,
                         plg->m_services->microhttphost(),
                         plg->m_services->microhttpport(),
-                        plg->m_services->getpathprefix(plg))) {
+                        plg->m_services->getpathprefix(plg->m_name),
+                        plg->m_services->getupnpaddr(),
+                        plg->m_services->getupnpport()
+            )) {
         LOGDEB1("PlgWithSlave::maybeStartCmd: startPluginCmd failed\n");
         return false;
     }
@@ -243,17 +284,14 @@ bool PlgWithSlave::Internal::maybeStartCmd()
         LockableShmSeg::Accessor access(seg);
         char *cp = (char *)(access.getseg());
         string data(cp);
-        LOGDEB1("PlgWithSlave::maybeStartCmd: segment content [" << data <<
-                "]\n");
+        LOGDEB1("PlgWithSlave::maybeStartCmd: segment content [" << data << "]\n");
         ConfSimple credsconf(data, true);
         string user, password;
         if (credsconf.get(plg->m_name + "user", user) &&
             credsconf.get(plg->m_name + "pass", password)) {
             unordered_map<string,string> res;
-            if (!cmd.callproc("login", {{"user", user}, {"password", password}},
-                              res)) {
-                LOGINF("PlgWithSlave::maybeStartCmd: tried login but failed for "
-                       << plg->m_name);
+            if (!cmd.callproc("login", {{"user", user}, {"password", password}}, res)) {
+                LOGINF("PlgWithSlave::maybeStartCmd: tried login but failed for " << plg->m_name);
             }
         }
     } else {
@@ -276,9 +314,9 @@ bool PlgWithSlave::startInit()
 // The Python code calls the service to translate the trackid to a temp
 // URL. We cache the result for a few seconds to avoid multiple calls
 // to tidal.
-string PlgWithSlave::get_media_url(const string& path)
+string PlgWithSlave::get_media_url(const string& path, const std::string& useragent)
 {
-    LOGDEB0("PlgWithSlave::get_media_url: " << path << endl);
+    LOGDEB0("PlgWithSlave::get_media_url: " << path << "\n");
     if (!m->maybeStartCmd()) {
         return string();
     }
@@ -286,7 +324,7 @@ string PlgWithSlave::get_media_url(const string& path)
     if (m->laststream.path.compare(path) ||
         (now - m->laststream.opentime > 10)) {
         unordered_map<string, string> res;
-        if (!m->cmd.callproc("trackuri", {{"path", path}}, res)) {
+        if (!m->cmd.callproc("trackuri", {{"path", path}, {"user-agent", useragent}}, res)) {
             LOGERR("PlgWithSlave::get_media_url: slave failure\n");
             return string();
         }
@@ -331,7 +369,7 @@ static void catstring(string& dest, const string& s2)
     if (dest.empty()) {
         dest = s2;
     } else {
-        dest += string(" ") + s2;
+        dest += string(", ") + s2;
     }
 }
 
@@ -365,98 +403,105 @@ static void decodeResource(const Json::Value entry, UpSong::Res &res)
     }
 }
 
-#define JSONTOUPS(fld, nm) {catstring(song.fld, \
-                                      decoded[i].get(#nm, "").asString());}
+#define JSONTOUPS(fld, nm) {catstring(song.fld, decod_i.get(#nm, "").asString());}
 
-static int resultToEntries(const string& encoded, vector<UpSong>& entries)
+static int resultToEntries(const string& encoded, vector<UpSong>& entries,
+                           const std::string& classfilter = std::string())
 {
     Json::Value decoded;
     istringstream input(encoded);
     input >> decoded;
     LOGDEB0("PlgWithSlave::results: got " << decoded.size() << " entries \n");
-    LOGDEB1("PlgWithSlave::results: undecoded: " << decoded.dump() << endl);
 
     entries.reserve(decoded.size());
     for (unsigned int i = 0; i < decoded.size(); i++) {
-        entries.emplace_back();
-        UpSong& song = entries.back();
+        UpSong song;
+        Json::Value& decod_i = decoded[i];
+
+        // Possibly extract our vendor extension fields
+        for(auto it = decod_i.begin(); it != decod_i.end(); it++) {
+            if (beginswith(it.key().asString(), "upmpd:")) {
+                if (song.upmpfields == nullptr)
+                    song.upmpfields = new std::unordered_map<std::string, std::string>;
+                LOGDEB1("resultToEntries: "<<it.key().asString()<<" -> "<<(*it).asString()<<'\n');
+                auto& flds(*(song.upmpfields));
+                flds[it.key().asString()] = (*it).asString();
+            }
+        }
+
         JSONTOUPS(id, id);
         JSONTOUPS(parentid, pid);
         JSONTOUPS(title, tt);
         JSONTOUPS(artUri, upnp:albumArtURI);
         JSONTOUPS(artist, upnp:artist);
+        auto cr = decod_i.get("dc:creator", "").asString();
+        if (cr != song.artist) {
+            catstring(song.artist, cr);
+        }
         JSONTOUPS(upnpClass, upnp:class);
-        JSONTOUPS(date, dc:date)
-        JSONTOUPS(date, releasedate)
+        JSONTOUPS(dcdescription, dc:description);
+        JSONTOUPS(album, upnp:album);
+        JSONTOUPS(dcdate, dc:date);
+        JSONTOUPS(genre, upnp:genre);
+        song.didlfrag = decod_i.get("didlfrag", "").asString();
+    
         // tp is container ("ct") or item ("it")
-        string stp = decoded[i].get("tp", "").asString();
+        string stp = decod_i.get("tp", "").asString();
         if (!stp.compare("ct")) {
             song.iscontainer = true;
-            string ss = decoded[i].get("searchable", "").asString();
+            string ss = decod_i.get("searchable", "").asString();
             if (!ss.empty()) {
                 song.searchable = stringToBool(ss);
             }
         } else  if (!stp.compare("it")) {
             song.iscontainer = false;
-            JSONTOUPS(artist, dc:creator);
-            JSONTOUPS(genre, upnp:genre);
-            JSONTOUPS(album, upnp:album);
             JSONTOUPS(tracknum, upnp:originalTrackNumber);
             // Decode resource data in base record
-            decodeResource(decoded[i], song.rsrc);
+            decodeResource(decod_i, song.rsrc);
             // Possibly add resources from resource array if present
-            const Json::Value &resources = decoded[i]["resources"];
-            LOGDEB1("decoded['resources'] is type " << resources.type() << endl);
+            const Json::Value &resources = decod_i["resources"];
+            LOGDEB1("decoded['resources'] is type " << resources.type() << "\n");
             if (resources.isArray()) {
+                if (nullptr == song.resources) {
+                    song.resources = new std::vector<UpSong::Res>;
+                }
                 for (unsigned int i = 0; i < resources.size(); i++) {
-                    song.resources.push_back(UpSong::Res());
-                    decodeResource(resources[i], song.resources.back());
+                    song.resources->push_back(UpSong::Res());
+                    decodeResource(resources[i], song.resources->back());
                 }
             }
         } else {
-            LOGERR("PlgWithSlave::result: bad type in entry: " << stp <<
-                   "(title: " << song.title << ")\n");
+            LOGERR("PlgWithSlave::result: bad type: <" << stp << "> (titl: " << song.title << ")\n");
             continue;
         }
-        LOGDEB1("PlgWitSlave::result: pushing: " << song.dump() << endl);
+        if (!classfilter.empty() && song.upnpClass.find(classfilter) != 0) {
+            LOGDEB1("PlgWithSlave::resultToEntries: class mismatch " << classfilter << "\n");
+            continue;
+        }
+        LOGDEB1("PlgWithSlave::resultToEntries: pushing: " << song.dump() << "\n");
+        entries.push_back(song);
     }
     return decoded.size();
 }
 
 
-class ContentCacheEntry {
-public:
-    int toResult(const string& classfilter, int stidx, int cnt,
-                 vector<UpSong>& entries) const;
-    time_t m_time{time(0)};
-    int m_offset{0};
-    int m_total{-1};
-    vector<UpSong> m_results;
-};
-
-int ContentCacheEntry::toResult(const string& classfilter, int stidx, int cnt,
-                                vector<UpSong>& entries) const
+int ContentCacheEntry::toResult(int stidx, int cnt, vector<UpSong>& entries) const
 {
-    LOGDEB0("searchCacheEntryToResult: filter " << classfilter << " start " <<
+    LOGDEB0("searchCacheEntryToResult: start " <<
             stidx << " cnt " << cnt << " m_offset " << m_offset <<
             " m_results.size " << m_results.size() << "\n");
 
     if (stidx < m_offset) {
         // we're missing a part
-        LOGERR("ContentCacheEntry::toResult: stidx " << stidx << " < offset " <<
-               m_offset << "\n");
+        LOGERR("ContentCacheEntry::toResult: stidx " << stidx << " < offset " << m_offset << "\n");
         return -1;
     }
 
     if (cnt > 0)
         entries.reserve(cnt);
     for (int i = stidx - m_offset; i < int(m_results.size()); i++) {
-        if (!classfilter.empty() &&
-            m_results[i].upnpClass.find(classfilter) != 0) {
-            continue;
-        }
         LOGDEB1("ContentCacheEntry::toResult: pushing class " <<
-                m_results[i].upnpClass << " tt " << m_results[i].title << endl);
+                m_results[i].upnpClass << " tt " << m_results[i].title << "\n");
         entries.push_back(m_results[i]);
         if (cnt > 0 && int(entries.size()) >= cnt) {
             break;
@@ -467,19 +512,6 @@ int ContentCacheEntry::toResult(const string& classfilter, int stidx, int cnt,
     return m_total == -1 ? m_results.size() : m_total;
 }
 
-class ContentCache {
-public:
-    ContentCache(int retention_secs = 300)
-        : m_retention_secs(retention_secs) {}
-    std::shared_ptr<ContentCacheEntry> get(const string& query);
-    ContentCacheEntry& set(const string& query, ContentCacheEntry &entry);
-    void purge();
-private:
-    time_t m_lastpurge{time(0)};
-    int m_retention_secs;
-    unordered_map<string, ContentCacheEntry> m_cache;
-};
-
 void ContentCache::purge()
 {
     time_t now(time(0));
@@ -488,7 +520,7 @@ void ContentCache::purge()
     }
     for (auto it = m_cache.begin(); it != m_cache.end(); ) {
         if (now - it->second.m_time > m_retention_secs) {
-            LOGDEB0("ContentCache::purge: erasing " << it->first << endl);
+            LOGDEB0("ContentCache::purge: erasing " << it->first << "\n");
             it = m_cache.erase(it);
         } else {
             it++;
@@ -497,19 +529,19 @@ void ContentCache::purge()
     m_lastpurge = now;
 }
 
-std::shared_ptr<ContentCacheEntry> ContentCache::get(const string& key)
+std::unique_ptr<ContentCacheEntry> ContentCache::get(const string& key)
 {
     purge();
     auto it = m_cache.find(key);
     if (it != m_cache.end()) {
         ContentCacheEntry& entry = it->second;
         LOGDEB0("ContentCache::get: found " << key << " offset " << entry.m_offset <<
-            " count " << entry.m_results.size() << "\n");
+                " count " << entry.m_results.size() << "\n");
         // we return a copy of the vector. Make our multi-access life simpler...
-        return std::make_shared<ContentCacheEntry>(it->second);
+        return std::make_unique<ContentCacheEntry>(it->second);
     }
-    LOGDEB0("ContentCache::get: not found " << key << endl);
-    return std::shared_ptr<ContentCacheEntry>();
+    LOGDEB0("ContentCache::get: not found " << key << "\n");
+    return std::unique_ptr<ContentCacheEntry>();
 }
 
 ContentCacheEntry& ContentCache::set(const string& key, ContentCacheEntry &entry)
@@ -520,25 +552,17 @@ ContentCacheEntry& ContentCache::set(const string& key, ContentCacheEntry &entry
     return m_cache[key];
 }
 
-// Cache for searches
-static ContentCache o_scache(300);
-// Cache for browsing
-static ContentCache o_bcache(180);
-
 // Better return a bogus informative entry than an outright error:
 static int errorEntries(const string& pid, vector<UpSong>& entries)
 {
-    entries.push_back(
-        UpSong::item(pid + "$bogus", pid,
-                     "Service login or communication failure"));
+    entries.push_back(UpSong::item(pid + "$bogus", pid, "Service login or communication failure"));
     return 1;
 }
 
-int PlgWithSlave::browse(const string& objid, int stidx, int cnt,
-                         vector<UpSong>& entries,
-                         const vector<string>& sortcrits,
-                         BrowseFlag flg)
+int PlgWithSlave::browse(const string& objid, int stidx, int cnt, vector<UpSong>& entries,
+                         const vector<string>& sortcrits, BrowseFlag flg)
 {
+    std::unique_lock<std::mutex> lock(m->mutex);
     LOGDEB("PlgWithSlave::browse: offset " << stidx << " cnt " << cnt << "\n");
     entries.clear();
     if (!m->maybeStartCmd()) {
@@ -558,18 +582,17 @@ int PlgWithSlave::browse(const string& objid, int stidx, int cnt,
     string cachekey(m_name + ":" + objid);
     if (flg == CDPlugin::BFChildren) {
         // Check cache
-        auto cep = o_bcache.get(cachekey);
+        auto cep = m->bcache.get(cachekey);
         if (cep) {
-            LOGDEB("PlgWithSlave::browse: found cache entry: offset " <<
-                   cep->m_offset << " count " << cep->m_results.size() <<
-                   " total " << cep->m_total << "\n");
+            LOGDEB("PlgWithSlave::browse: found cache entry: offset " << cep->m_offset <<
+                   " count " << cep->m_results.size() << " total " << cep->m_total << "\n");
             if (cep->m_total > 0 && cnt + stidx > cep->m_total) {
                 cnt = cep->m_total - stidx;
                 LOGDEB("PlgWithSlave::browse: adjusted cnt to " << cnt << "\n");
             }
             if (cep->m_offset <= stidx &&
                 int(cep->m_results.size()) - (stidx - cep->m_offset) >= cnt) {
-                return cep->toResult("", stidx, cnt, entries);
+                return cep->toResult(stidx, cnt, entries);
             }
         }
     }
@@ -577,9 +600,8 @@ int PlgWithSlave::browse(const string& objid, int stidx, int cnt,
     std::string soffs = lltodecstr(stidx);
     std::string scnt = lltodecstr(cnt);
     unordered_map<string, string> res;
-    if (!m->cmd.callproc(
-            "browse", {{"objid", objid}, {"flag", sbflg},
-                       {"offset", soffs}, {"count", scnt}}, res)) {
+    if (!m->cmd.callproc("browse", {{"objid", objid}, {"flag", sbflg},
+                                    {"offset", soffs}, {"count", scnt}}, res)) {
         LOGERR("PlgWithSlave::browse: slave failure\n");
         return errorEntries(objid, entries);
     }
@@ -609,61 +631,49 @@ int PlgWithSlave::browse(const string& objid, int stidx, int cnt,
     
     if (flg == CDPlugin::BFChildren) {
         ContentCacheEntry entry;
-        ContentCacheEntry& e = nocache ? entry : o_bcache.set(cachekey, entry);
+        ContentCacheEntry& e = nocache ? entry : m->bcache.set(cachekey, entry);
         e.m_offset = resoffs;
         e.m_total = total;
         resultToEntries(ite->second, e.m_results);
-        return e.toResult("", stidx, cnt, entries);
+        return e.toResult(stidx, cnt, entries);
     } else {
         return resultToEntries(ite->second, entries);
     }
 }
 
-int PlgWithSlave::search(const string& ctid, int stidx, int cnt,
-                         const string& searchstr,
-                         vector<UpSong>& entries,
-                         const vector<string>& sortcrits)
+// Computing a pre-cooked query for simple-minded plugins.
+//
+// The plugins also receive the original search string, and, e.g. Uprcl, uses it instead of the
+// simplified parameters.
+//
+// Note that none of the qobuz/gmusic/tidal plugins actually use the slavefield part (defining in
+// what field the term should match).
+// 
+// Ok, so the upnp query language is quite powerful, but most our plugins, not so much. We prepare a
+// simplified search by getting rid of parenthesis and then trying to find the first searchExp on a
+// field we can handle, pretend the operator is "contains" and just do it. I so don't want to
+// implement a parser for the query language when the services don't support anything complicated
+// anyway.
+static bool eli5(const std::string& searchstr, std::string& slavefield, std::string& value,
+                 std::string& classfilter, std::string& objkind)
 {
-    LOGDEB("PlgWithSlave::search: [" << searchstr << "]\n");
-    entries.clear();
-    if (!m->maybeStartCmd()) {
-        return errorEntries(ctid, entries);
-    }
-
-    // Computing a pre-cooked query. For simple-minded plugins.
-    // Note that none of the qobuz/gmusic/tidal plugins actually use
-    // the slavefield part (defining in what field the term should
-    // match).
-    // 
-    // Ok, so the upnp query language is quite powerful, but us, not
-    // so much. We get rid of parenthesis and then try to find the
-    // first searchExp on a field we can handle, pretend the operator
-    // is "contains" and just do it. I so don't want to implement a
-    // parser for the query language when the services don't support
-    // anything complicated anyway, and the users don't even want it...
     string ss;
     neutchars(searchstr, ss, "()");
-
-    // The search had better be space-separated. no
-    // upnp:artist="beatles" for you
+    // The search had better be space-separated. no upnp:artist="beatles" for you
     vector<string> vs;
     stringToStrings(ss, vs);
-
-    // The sequence can now be either [field, op, value], or
-    // [field, op, value, and/or, field, op, value,...]
+    // The sequence can now be either [field, op, value], or [field, op, value, and/or, field, op,
+    // value,...]. The number of fields is n*4 - 1 (missing last conjunction)
     if ((vs.size() + 1) % 4 != 0) {
-        LOGERR("PlgWithSlave::search: bad search string: [" << searchstr <<
-               "]\n");
-        return errorEntries(ctid, entries);
+        LOGERR("PlgWithSlave::search: bad search string: [" << searchstr << "]\n");
+        return false;
     }
-    string slavefield;
-    string value;
-    string classfilter;
-    string objkind;
+
+    // Note that if we only keep one object kind filtering clause and one content filtering one. If
+    // there are more, these will be the last ones.
     for (unsigned int i = 0; i < vs.size()-2; i += 4) {
         const string& upnpproperty = vs[i];
-        LOGDEB("PlgWithSlave::search:clause: " << vs[i] << " " << vs[i+1] <<
-               " " << vs[i+2] << endl);
+        LOGDEB("PlgWithSlave::search:clause: " << vs[i] << " " << vs[i+1] << " " << vs[i+2] << "\n");
         if (!upnpproperty.compare("upnp:class")) {
             // This defines -what- we are looking for (track/album/artist)
             const string& what(vs[i+2]);
@@ -679,27 +689,41 @@ int PlgWithSlave::search(const string& ctid, int stidx, int cnt,
                 objkind = "playlist";
             }
             classfilter = what;
-        } else if (!upnpproperty.compare("upnp:artist") ||
-                   !upnpproperty.compare("dc:author")) {
+        } else if (!upnpproperty.compare("upnp:artist") || !upnpproperty.compare("dc:author")) {
             slavefield = "artist";
             value = vs[i+2];
-            break;
         } else if (!upnpproperty.compare("upnp:album")) {
             slavefield = "album";
             value = vs[i+2];
-            break;
         } else if (!upnpproperty.compare("dc:title")) {
             slavefield = "track";
             value = vs[i+2];
-            break;
         }
     }
+    return true;
+}
+
+
+int PlgWithSlave::search(const string& ctid, int stidx, int cnt, const string& searchstr,
+                         vector<UpSong>& entries, const vector<string>& sortcrits)
+{
+    std::unique_lock<std::mutex> lock(m->mutex);
+    LOGDEB("PlgWithSlave::search: [" << searchstr << "]\n");
+    entries.clear();
+    if (!m->maybeStartCmd()) {
+        return errorEntries(ctid, entries);
+    }
+
+    string slavefield, value, classfilter, objkind;
+    if (!eli5(searchstr, slavefield, value, classfilter, objkind)) {
+        return errorEntries(ctid, entries);
+    }        
 
     // In cache ?
     string cachekey(m_name + ":" + ctid + ":" + searchstr);
-    auto cep = o_scache.get(cachekey);
+    auto cep = m->scache.get(cachekey);
     if (cep) {
-        int total = cep->toResult(classfilter, stidx, cnt, entries);
+        int total = cep->toResult(stidx, cnt, entries);
         return total;
     }
 
@@ -742,9 +766,9 @@ int PlgWithSlave::search(const string& ctid, int stidx, int cnt,
     }
     // Convert the whole set and store in cache
     ContentCacheEntry entry;
-    ContentCacheEntry& e = nocache ? entry : o_scache.set(cachekey, entry);
+    ContentCacheEntry& e = nocache ? entry : m->scache.set(cachekey, entry);
     e.m_offset = resoffs;
     e.m_total = total;
-    resultToEntries(ite->second, e.m_results);
-    return e.toResult(classfilter, stidx, cnt, entries);
+    resultToEntries(ite->second, e.m_results, classfilter);
+    return e.toResult(stidx, cnt, entries);
 }
