@@ -1,4 +1,4 @@
-/* Copyright (C) 2016 J.F.Dockes
+/* Copyright (C) 2016-2023 J.F.Dockes
  *       This program is free software; you can redistribute it and/or modify
  *       it under the terms of the GNU Lesser General Public License as published by
  *       the Free Software Foundation; either version 2.1 of the License, or
@@ -14,17 +14,13 @@
  *       Free Software Foundation, Inc.,
  *       59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
-
 #include "config.h"
 
 #include "contentdirectory.hxx"
 
 #include <functional>
-#include <iostream>
-#include <map>
 #include <utility>
 #include <unordered_map>
-#include <sstream>
 
 #include "libupnpp/log.hxx"
 #include "libupnpp/soaphelp.hxx"
@@ -44,7 +40,7 @@ using namespace UPnPProvider;
 class ContentDirectory::Internal {
 public:
     Internal (ContentDirectory *sv, MediaServer *dv)
-        : service(sv), msdev(dv), updateID("1") { }
+        : service(sv), msdev(dv), updateID("1") {}
 
     ~Internal() {
         for (auto& it : plugins) {
@@ -52,11 +48,10 @@ public:
         }
     }
 
-    // Start plugins which have long init so that the user has less to
-    // wait on first access
+    // Start plugins which have long init so that the user has less to wait on first access
     void maybeStartSomePlugins(bool enabled);
 
-    void maybeInitUpnpHost() {
+    void maybeInit() {
         if (upnphost.empty()) {
             UpnpDevice *dev;
             if (!service || !(dev = service->getDevice())) {
@@ -69,15 +64,15 @@ public:
                 return;
             }
             upnpport = usport;
-            LOGDEB("ContentDirectory: upnphost "<< upnphost <<
-                   " upnpport " << upnpport << endl);
+            getOptionValue("msrootalias", rootalias);
+            LOGDEB("ContentDirectory: upnphost ["<< upnphost << "] upnpport [" << upnpport <<
+                   "] rootalias [" << rootalias << "]\n");
         }
     }
     
     CDPlugin *pluginFactory(const string& appname) {
-        LOGDEB("ContentDirectory::pluginFactory: for " << appname << endl);
-
-        maybeInitUpnpHost();
+        LOGDEB("ContentDirectory::pluginFactory: for " << appname << "\n");
+        maybeInit();
         return new PlgWithSlave(appname, service);
     }
 
@@ -86,11 +81,7 @@ public:
         if (it != plugins.end()) {
             return it->second;
         } else {
-            CDPlugin *plug = pluginFactory(appname);
-            if (plug) {
-                plugins[appname] = plug;
-            }
-            return plug;
+            return plugins[appname] = pluginFactory(appname);
         }
     }
 
@@ -99,17 +90,15 @@ public:
     unordered_map<string, CDPlugin *> plugins;
     string upnphost;
     int upnpport;
+    string rootalias;
     string updateID;
 };
 
-static const string
-sTpContentDirectory("urn:schemas-upnp-org:service:ContentDirectory:1");
-static const string
-sIdContentDirectory("urn:upnp-org:serviceId:ContentDirectory");
+static const string sTpContentDirectory("urn:schemas-upnp-org:service:ContentDirectory:1");
+static const string sIdContentDirectory("urn:upnp-org:serviceId:ContentDirectory");
 
 ContentDirectory::ContentDirectory(MediaServer *dev, bool enabled)
-    : UpnpService(sTpContentDirectory, sIdContentDirectory,
-                  "ContentDirectory.xml", dev),
+    : UpnpService(sTpContentDirectory, sIdContentDirectory, "ContentDirectory.xml", dev),
       m(new Internal(this, dev))
 {
     dev->addActionMapping(
@@ -137,17 +126,16 @@ ContentDirectory::~ContentDirectory()
 
 int ContentDirectory::actGetSearchCapabilities(const SoapIncoming& sc, SoapOutgoing& data)
 {
-    LOGDEB("ContentDirectory::actGetSearchCapabilities: " << endl);
+    LOGDEB("ContentDirectory::actGetSearchCapabilities: " << "\n");
 
-    std::string out_SearchCaps(
-        "upnp:class,upnp:artist,dc:creator,upnp:album,dc:title");
+    std::string out_SearchCaps("upnp:class,upnp:artist,dc:creator,upnp:album,dc:title");
     data.addarg("SearchCaps", out_SearchCaps);
     return UPNP_E_SUCCESS;
 }
 
 int ContentDirectory::actGetSortCapabilities(const SoapIncoming& sc, SoapOutgoing& data)
 {
-    LOGDEB("ContentDirectory::actGetSortCapabilities: " << endl);
+    LOGDEB("ContentDirectory::actGetSortCapabilities: " << "\n");
 
     std::string out_SortCaps;
     data.addarg("SortCaps", out_SortCaps);
@@ -156,7 +144,7 @@ int ContentDirectory::actGetSortCapabilities(const SoapIncoming& sc, SoapOutgoin
 
 int ContentDirectory::actGetSystemUpdateID(const SoapIncoming& sc, SoapOutgoing& data)
 {
-    LOGDEB("ContentDirectory::actGetSystemUpdateID: " << endl);
+    LOGDEB("ContentDirectory::actGetSystemUpdateID: " << "\n");
 
     std::string out_Id = m->updateID;
     data.addarg("Id", out_Id);
@@ -171,8 +159,7 @@ static bool makerootdir()
     string reason;
     set<string> entries;
     if (!listdir(pathplg, reason, entries)) {
-        LOGERR("ContentDirectory::makerootdir: can't read " << pathplg <<
-               " : " << reason << endl);
+        LOGERR("ContentDirectory::makerootdir: can't read " << pathplg << " : " << reason << "\n");
         return false;
     }
 
@@ -182,8 +169,8 @@ static bool makerootdir()
         }
         string userkey = entry + "user";
         string autostartkey = entry + "autostart";
-        if (!g_config->hasNameAnywhere(userkey) &&
-            !g_config->hasNameAnywhere(autostartkey)) {
+        std::string v;
+        if (!getOptionValue(userkey, v) && !getOptionValue(autostartkey, v)) {
             LOGINF("ContentDirectory: not creating entry for " << entry <<
                    " because neither " << userkey << " nor " << autostartkey <<
                    " are defined in the configuration\n");
@@ -194,16 +181,16 @@ static bool makerootdir()
         // we compute a title (to be displayed in the root directory)
         // from the plugin name.
         string title;
-        if (!g_config->get(entry + "title", title)) {
-            title = stringtoupper((const string&)entry.substr(0,1)) +
-                entry.substr(1, entry.size()-1);
+        if (!getOptionValue(entry + "title", title)) {
+            title = stringtoupper((const string&)entry.substr(0, 1)) +
+                entry.substr(1, entry.size() - 1);
         }
         rootdir.push_back(UpSong::container("0$" + entry + "$", "0", title));
     }
 
     if (rootdir.empty()) {
-        // This should not happen, as we only start the CD if services
-        // are configured !
+        // Just as a precaution. The rootdir should never be used in this case because the media
+        // server will not be started.
         rootdir.push_back(UpSong::item("0$none$", "0", "No services found"));
         return false;
     } else {
@@ -219,7 +206,7 @@ bool ContentDirectory::mediaServerNeeded()
 // Returns totalmatches
 static size_t readroot(int offs, int cnt, vector<UpSong>& out)
 {
-    //LOGDEB("readroot: offs " << offs << " cnt " << cnt << endl);
+    //LOGDEB("readroot: offs " << offs << " cnt " << cnt << "\n");
     if (rootdir.empty()) {
         makerootdir();
     }
@@ -258,6 +245,14 @@ static string appForId(const string& id)
     return id.substr(dol0 + 1, dol1 - dol0 -1);
 }
 
+std::string CDPluginServices::pluginRootFromObjid(const std::string& objid)
+{
+    auto app = appForId(objid);
+    if (app.empty()) { // ??
+        return "0";
+    } 
+    return std::string("0$") + app + "$";
+}
 
 void ContentDirectory::Internal::maybeStartSomePlugins(bool enabled)
 {
@@ -278,9 +273,8 @@ void ContentDirectory::Internal::maybeStartSomePlugins(bool enabled)
     
     for (auto& entry : rootdir) {
         string app = appForId(entry.id);
-        if (g_config->getBool(app + "autostart", false)) {
-            LOGDEB0("ContentDirectory::Internal::maybeStartSomePlugins: "
-                    "starting " << app << endl);
+        if (getBoolOptionValue(app + "autostart", false)) {
+            LOGDEB0("ContentDirectory::Internal::maybeStartSomePlugins: starting " << app << "\n");
             CDPlugin *p = pluginForApp(app);
             if (p) {
                 p->startInit();
@@ -289,22 +283,25 @@ void ContentDirectory::Internal::maybeStartSomePlugins(bool enabled)
     }
 }
 
-// Really preposterous: bubble (and maybe others) searches in root,
-// but we can't do this. So memorize the last browsed object ID and
-// use this as a replacement when root search is requested. Forget
-// about multiaccess, god forbid multithreading etc. Will work in most
-// cases though :)
+// Really preposterous: bubble (and maybe others) searches in root, but we can't do this. So
+// memorize the last browsed object ID and use this as a replacement when root search is
+// requested. Forget about multiaccess, god forbid multithreading etc. Will work in most cases
+// though :)
 static string last_objid;
 
 int ContentDirectory::actBrowse(const SoapIncoming& sc, SoapOutgoing& data)
 {
     bool ok = false;
     std::string in_ObjectID;
+       
     ok = sc.get("ObjectID", &in_ObjectID);
     if (!ok) {
         LOGERR("ContentDirectory::actBrowse: no ObjectID in params\n");
         return UPNP_E_INVALID_PARAM;
     }
+    if (!m->rootalias.empty() && in_ObjectID.find(m->rootalias) != 0) {
+        in_ObjectID = m->rootalias + in_ObjectID.substr(1);
+    }
     std::string in_BrowseFlag;
     ok = sc.get("BrowseFlag", &in_BrowseFlag);
     if (!ok) {
@@ -340,7 +337,7 @@ int ContentDirectory::actBrowse(const SoapIncoming& sc, SoapOutgoing& data)
            " BrowseFlag " << in_BrowseFlag << " Filter " << in_Filter <<
            " StartingIndex " << in_StartingIndex <<
            " RequestedCount " << in_RequestedCount <<
-           " SortCriteria " << in_SortCriteria << endl);
+           " SortCriteria " << in_SortCriteria << "\n");
 
     last_objid = in_ObjectID;
     
@@ -392,7 +389,7 @@ int ContentDirectory::actBrowse(const SoapIncoming& sc, SoapOutgoing& data)
         out_Result += entries[i].didl();
     } 
     out_Result += tailDIDL();
-    LOGDEB1("ContentDirectory::Browse: didl: " << out_Result << endl);
+    LOGDEB1("ContentDirectory::Browse: didl: " << out_Result << "\n");
     
     data.addarg("Result", out_Result);
     LOGDEB1("ContentDirectory::actBrowse: result [" << out_Result << "]\n");
@@ -411,6 +408,9 @@ int ContentDirectory::actSearch(const SoapIncoming& sc, SoapOutgoing& data)
         LOGERR("ContentDirectory::actSearch: no ContainerID in params\n");
         return UPNP_E_INVALID_PARAM;
     }
+    if (!m->rootalias.empty() && in_ContainerID.find(m->rootalias) != 0) {
+        in_ContainerID = m->rootalias + in_ContainerID.substr(1);
+    }
     std::string in_SearchCriteria;
     ok = sc.get("SearchCriteria", &in_SearchCriteria);
     if (!ok) {
@@ -447,7 +447,7 @@ int ContentDirectory::actSearch(const SoapIncoming& sc, SoapOutgoing& data)
            " SearchCriteria " << in_SearchCriteria <<
            " Filter " << in_Filter << " StartingIndex " << in_StartingIndex <<
            " RequestedCount " << in_RequestedCount <<
-           " SortCriteria " << in_SortCriteria << endl);
+           " SortCriteria " << in_SortCriteria << "\n");
 
     vector<string> sortcrits;
     stringToStrings(in_SortCriteria, sortcrits);
@@ -460,13 +460,19 @@ int ContentDirectory::actSearch(const SoapIncoming& sc, SoapOutgoing& data)
     // Go fetch
     vector<UpSong> entries;
     size_t totalmatches = 0;
-    if (!in_ContainerID.compare("0")) {
-        // Root directory: can't search in there: we don't know what
-        // plugin to pass the search to. Substitute last browsed. Yes
-        // it does break in multiuser mode, and yes it's preposterous.
-        LOGERR("ContentDirectory::actSearch: Can't search in root. "
-               "Substituting last browsed container\n");
-        in_ContainerID = last_objid;
+    if (in_ContainerID == "0") {
+        // Root directory: can't search in there: we don't know what plugin to pass the search to
+        // (and we don't want to search all). Substitute last browsed. Yes it does break in
+        // multiuser mode, and yes it's preposterous.
+        in_ContainerID = pluginRootFromObjid(last_objid);
+        if (in_ContainerID == "0") {
+            LOGERR("ContentDirectory::actSearch: CP requested search in root and could not find "
+                   "plugin from last browsed container\n");
+            // Fallthrough, appForId is going to fail just below.
+        } else {
+            LOGINF("ContentDirectory::actSearch: CP requested search in global root: substituting "
+                   "plugin root [" << in_ContainerID << "] from last browsed container\n");
+        }
     }
 
     // Pass off request to appropriate app, defined by 1st elt in id
@@ -474,8 +480,7 @@ int ContentDirectory::actSearch(const SoapIncoming& sc, SoapOutgoing& data)
     CDPlugin *plg = m->pluginForApp(app);
     if (plg) {
         totalmatches = plg->search(in_ContainerID, in_StartingIndex,
-                                   in_RequestedCount, in_SearchCriteria,
-                                   entries, sortcrits);
+                                   in_RequestedCount, in_SearchCriteria, entries, sortcrits);
     } else {
         LOGERR("ContentDirectory::Search: unknown app: [" << app << "]\n");
         return UPNP_E_INVALID_PARAM;
@@ -487,6 +492,7 @@ int ContentDirectory::actSearch(const SoapIncoming& sc, SoapOutgoing& data)
     out_UpdateID = m->updateID;
     out_Result = headDIDL();
     for (unsigned int i = 0; i < entries.size(); i++) {
+        LOGDEB1("Search result title: " << entries[i].title << "\n");
         out_Result += entries[i].didl();
     } 
     out_Result += tailDIDL();
@@ -495,6 +501,8 @@ int ContentDirectory::actSearch(const SoapIncoming& sc, SoapOutgoing& data)
     data.addarg("NumberReturned", out_NumberReturned);
     data.addarg("TotalMatches", out_TotalMatches);
     data.addarg("UpdateID", out_UpdateID);
+    LOGDEB0("ContentDirectory::actSearch: " << " SearchCriteria " << in_SearchCriteria <<
+            " returns " << out_NumberReturned << " results\n");
     return UPNP_E_SUCCESS;
 }
 
@@ -526,13 +534,13 @@ CDPlugin *ContentDirectory::getpluginforpath(const string& path)
     return m->pluginForApp(app);
 }
 
-std::string ContentDirectory::getupnpaddr(CDPlugin *)
+std::string ContentDirectory::getupnpaddr()
 {
     return m->upnphost;
 }
 
 
-int ContentDirectory::getupnpport(CDPlugin *)
+int ContentDirectory::getupnpport()
 {
     return m->upnpport;
 }
@@ -542,22 +550,21 @@ std::string ContentDirectory::getfname()
     return m->msdev->getfname();
 }
 
-// Note that this is not needed by ohcredentials (the slave script
-// does not generate URLs in this case, and the mhttp servers listens
-// on all addresses).
+// Note that this is not needed by ohcredentials (the slave script does not generate URLs in this
+// case, and the mhttp servers listens on all addresses).
 string ContentDirectory::microhttphost()
 {
     string host;
-    if (g_config && g_config->get("plgmicrohttphost", host) && !host.empty()) {
-        LOGDEB("ContentDirectory::microhttphost: from config:" << host << endl);
+    if (getOptionValue("plgmicrohttphost", host) && !host.empty()) {
+        LOGDEB("ContentDirectory::microhttphost: from config:" << host << "\n");
         return host;
     }
-    m->maybeInitUpnpHost();
+    m->maybeInit();
     return m->upnphost;
 }
 
 // Static for use needed by ohcredentials
 int CDPluginServices::microhttpport()
 {
-    return g_config->getInt("plgmicrohttpport", 49149);
+    return getIntOptionValue("plgmicrohttpport", 49149);
 }
