@@ -234,8 +234,10 @@ UpMpd::UpMpd(const string& hwaddr, const string& friendlyname,
     
     if (0 == (opts.options & upmpdNoAV)) {
         std::string avfname{friendlyname + "-UPnP/AV"};
-        g_config->get("avfriendlyname", avfname);
-        // Add bogus string to avfname in case user set it same as fname
+        if (getOptionValue("avfriendlyname", avfname, avfname)) {
+            avfname = fnameSetup(avfname);
+        }
+        // UUID: add bogus string to avfname in case user set it same as fname
         std::string deviceid =  std::string("uuid:") +
             LibUPnP::makeDevUUID(avfname + "xy3vhst39", hwaddr);
         m_av = new UpMpdMediaRenderer(this, deviceid, avfname);
@@ -313,9 +315,9 @@ bool UpMpd::checkContentFormat(const string& uri, const string& didl,
     
     const std::unordered_set<std::string>& supportedformats =
         Protocolinfo::the()->getsupportedformats();
-
+    auto decoded_uri = pc_decode(uri);
     for (const auto& resource : dobj.m_resources) {
-        if (!resource.m_uri.compare(uri)) {
+        if (pc_decode(resource.m_uri) == decoded_uri) {
             ProtocolinfoEntry e;
             if (!resource.protoInfo(e)) {
                 LOGERR("checkContentFormat: resource has no protocolinfo\n");
